/*jslint node: true, nomen: true*/
module.exports.name = 'expressApp';
module.exports.dependencies = [
    'expressSingleton',
    'express',
    'path',
    'favicon',
    'logger',
    'cookieParser',
    'bodyParser',
    'serveStatic',
    'less',
    'controllerFactory'
];
module.exports.factory = function (app, express, path, favicon, logger, cookieParser, bodyParser, serveStatic, less, controllerFactory) {
    "use strict";

    var before,
        after;
    
    before = function () {
        // view engine setup
        app.set('views', path.join(__dirname, 'views'));
        app.set('view engine', 'hbs');

        // uncomment after placing your favicon in /public
        //app.use(favicon(__dirname + '/public/favicon.ico'));
        app.use(logger('dev'));
        app.use(bodyParser.json());
        app.use(bodyParser.urlencoded({ extended: false }));
        app.use(cookieParser());
        app.use(less(path.join(__dirname, 'public')));
        app.use(serveStatic(path.join(__dirname, 'public')));
    };
    
    after = function () {
        // catch 404 and forward to error handler
        app.use(function (req, res, next) {
            var err = new Error('Not Found');
            err.status = 404;
            next(err);
        });

        // error handlers

        // development error handler
        // will print stacktrace
        if (app.get('env') === 'development') {
            app.use(function (err, req, res, next) {
                res.status(err.status || 500);
                res.render('error', {
                    message: err.message,
                    error: err
                });
            });
        }

        // production error handler
        // no stacktraces leaked to user
        app.use(function (err, req, res, next) {
            res.status(err.status || 500);
            res.render('error', {
                message: err.message,
                error: {}
            });
        });
    };
    
    before();
    controllerFactory(app, after);

    return app;
};
